#
#	FILE:	 Random_Maps.py
#	AUTHOR:  Shelly Crawford (Fallblau)
#	PURPOSE: Hacked version of Sirian's Tilted_Axis map modified in
#			an attempt to have truly random maps
#-----------------------------------------------------------------------------
#	Original script by Sirian Copyright (c) 2005 Firaxis Games, Inc. All rights reserved.
#-----------------------------------------------------------------------------
#
#  Modification Version .09 SC 03-NOV-2005
#  Modification Version .0901 SC 04-NOV-2005
#  Modification Version .0902 SC 06-NOV-2005
#

from CvPythonExtensions import *
import CvUtil
import CvMapGeneratorUtil
from CvMapGeneratorUtil import FractalWorld
from CvMapGeneratorUtil import TerrainGenerator
from CvMapGeneratorUtil import FeatureGenerator

#def isAdvancedMap():
#	"This map should show up in simple mode"
#	return 0

def getDescription():
	return "TXT_KEY_MAP_SCRIPT_RANDOM_MAPS_DESCR"

def getNumCustomMapOptions():
	return 1
	
def getCustomMapOptionName(argsList):
	translated_text = unicode(CyTranslator().getText("TXT_KEY_MAP_SCRIPT_LANDMASS_SIZE", ()))
	return translated_text
	
def getNumCustomMapOptionValues(argsList):
	return 5

	
def getCustomMapOptionDescAt(argsList):
	iSelection = argsList[1]
	selection_names = ["TXT_KEY_MAP_SCRIPT_MASSIVE_CONTINENTS",
	                   "TXT_KEY_MAP_SCRIPT_NORMAL_CONTINENTS",
	                   "TXT_KEY_MAP_SCRIPT_SMALL_CONTINENTS",
	                   "TXT_KEY_MAP_SCRIPT_ISLANDS",
	                   "TXT_KEY_MAP_SCRIPT_TINY_ISLANDS"
				]
	translated_text = unicode(CyTranslator().getText(selection_names[iSelection], ()))
	return translated_text
	
def getCustomMapOptionDefault(argsList):
	return -1

# SC 03-NOV-2005
# Remming this out as we want normal grid sizes
#def getGridSize(argsList):
#	"Override Grid Size function to make the maps square."
#	grid_sizes = {
#		WorldSizeTypes.WORLDSIZE_DUEL:      (8,8),
#		WorldSizeTypes.WORLDSIZE_TINY:      (10,10),
#		WorldSizeTypes.WORLDSIZE_SMALL:     (13,13),
#		WorldSizeTypes.WORLDSIZE_STANDARD:  (16,16),
#		WorldSizeTypes.WORLDSIZE_LARGE:     (20,20),
#		WorldSizeTypes.WORLDSIZE_HUGE:      (25,25)
#	}
#
#	if (argsList[0] == -1): # (-1,) is passed to function on loads
#		return []
#	[eWorldSize] = argsList
#	return grid_sizes[eWorldSize]

def getWrapX():
	# SC 03-NOV-2005
	# Modding to fix wrapping hopefully
	# return False
	return True
def getWrapY():
	# SC 03-NOV-2005
	# Modding to fix wrapping hopefully
	# return True
	return False
    
# subclass FractalWorld to enable square exponents for use with Random_Maps Axis.
class Random_MapsAxisFractalWorld(CvMapGeneratorUtil.FractalWorld):
	def initFractal(self, continent_grain = 2, rift_grain = 2, has_center_rift = True):
		"For no rifts, use rift_grain = -1"
		# Monkeying with this to see what it does...
		iFlags = CyFractal.FracVals.FRAC_WRAP_X + CyFractal.FracVals.FRAC_POLAR
		# SC 03-NOV-2005
		# Remming to unset world sizes
#		worldsizes = {
#			WorldSizeTypes.WORLDSIZE_DUEL:      (6,6),
#			WorldSizeTypes.WORLDSIZE_TINY:      (6,6),
#			WorldSizeTypes.WORLDSIZE_SMALL:     (6,6),
#			WorldSizeTypes.WORLDSIZE_STANDARD:  (7,7),
#			WorldSizeTypes.WORLDSIZE_LARGE:     (7,7),
#			WorldSizeTypes.WORLDSIZE_HUGE:      (7,7)
#			}
		# SC 04-NOV-2005
		# Hrmm... after remming this, I got basically the same map four times
		# in a row... Unremming to check...
		# OK, decided to set them along the lines of the ones shown
		# in Pangaea.py
		#worldsizes = {
		#	WorldSizeTypes.WORLDSIZE_DUEL:      (10,6),
		#	WorldSizeTypes.WORLDSIZE_TINY:      (13,8),
		#	WorldSizeTypes.WORLDSIZE_SMALL:     (16,10),
		#	WorldSizeTypes.WORLDSIZE_STANDARD:  (21,13),
		#	WorldSizeTypes.WORLDSIZE_LARGE:     (26,16),
		#	WorldSizeTypes.WORLDSIZE_HUGE:      (21,19)
		#	}
		# That seemed to cause problems... Trying something different
		worldsizes = {
			WorldSizeTypes.WORLDSIZE_DUEL:      (6,6),
			WorldSizeTypes.WORLDSIZE_TINY:      (7,7),
			WorldSizeTypes.WORLDSIZE_SMALL:     (8,8),
			WorldSizeTypes.WORLDSIZE_STANDARD:  (9,9),
			WorldSizeTypes.WORLDSIZE_LARGE:     (10,10),
			WorldSizeTypes.WORLDSIZE_HUGE:      (11,11)
			}
		(fracXExp, fracYExp) = worldsizes[self.map.getWorldSize()]

		if rift_grain >= 0:
			self.riftsFrac = CyFractal()
			self.riftsFrac.fracInit(self.iNumPlotsX, self.iNumPlotsY, rift_grain, self.mapRand, iFlags, fracXExp, fracYExp)
			if has_center_rift:
				iFlags = iFlags | CyFractal.FracVals.FRAC_CENTER_RIFT
			self.continentsFrac.fracInitRifts(self.iNumPlotsX, self.iNumPlotsY, continent_grain, self.mapRand, iFlags, self.riftsFrac, fracXExp, fracYExp)
		else:
			self.continentsFrac.fracInit(self.iNumPlotsX, self.iNumPlotsY, continent_grain, self.mapRand, iFlags, fracXExp, fracYExp)

def generatePlotTypes():
	gc = CyGlobalContext()
	map = CyMap()
	dice = gc.getGame().getMapRand()
	fractal_world = Random_MapsAxisFractalWorld()
	
	# Get user input.
	userInputLandmass = map.getCustomMapOption(0)
	
	if userInputLandmass == 4:
		NiTextOut("Setting Plot Types (Python Random_Maps Axis, Tiny Islands) ...")
		fractal_world.initFractal(continent_grain = 5, rift_grain = -1, has_center_rift = False)
		return fractal_world.generatePlotTypes(grain_amount = 4)

	elif userInputLandmass == 3:
		NiTextOut("Setting Plot Types (Python Random_Maps Axis, Islands) ...")
		fractal_world.initFractal(continent_grain = 4, rift_grain = -1, has_center_rift = False)
		return fractal_world.generatePlotTypes(grain_amount = 4)

	elif userInputLandmass == 2:
		NiTextOut("Setting Plot Types (Python Random_Maps Axis, Small Continents) ...")
		fractal_world.initFractal(continent_grain = 3, rift_grain = 3, has_center_rift = False)
		return fractal_world.generatePlotTypes(grain_amount = 4)
		
	elif userInputLandmass == 0:
		NiTextOut("Setting Plot Types (Python Random_Maps Axis, Massive Continents) ...")
		fractal_world.initFractal(continent_grain = 1, rift_grain = 2, has_center_rift = False)
		return fractal_world.generatePlotTypes(grain_amount = 4)

	else: # standard lands
		NiTextOut("Setting Plot Types (Python Random_Maps Axis, Normal Continents) ...")
		fractal_world.initFractal(continent_grain = 2, rift_grain = 2, has_center_rift = True)
		return fractal_world.generatePlotTypes(grain_amount = 4)
	
# subclass TerrainGenerator to make the climate "latitudes" run west to east
class Random_MapsAxisTerrainGenerator(CvMapGeneratorUtil.TerrainGenerator):
	def getLatitudeAtPlot(self, iX, iY):
		# SC 03-NOV-2005
		# Latitudes run horizontally as per regular for a world with a Random_Maps axis.
		# Changed this to match the one in CvMapGeneratorUtil
		lat = abs((self.iHeight / 2) - iY)/float(self.iHeight/2) # 0.0 = equator, 1.0 = pole

		# Adjust latitude using self.variation fractal, to mix things up:
		# This is the original line
		lat += (128 - self.variation.getHeight(iX, iY))/(255.0 * 5.0)
		# Experimenting
		# lat += (256 - self.variation.getHeight(iX, iY))/(255.0 * 5.0)

		# Limit to the range [0, 1]:
		if lat < 0:
			lat = 0.0
		if lat > 1:
			lat = 1.0

		return lat

def generateTerrainTypes():
	NiTextOut("Generating Terrain (Python Random_Maps Axis) ...")
	terraingen = Random_MapsAxisTerrainGenerator()
	terrainTypes = terraingen.generateTerrain()
	return terrainTypes

# subclass FeatureGenerator to make the climate "latitudes" run west to east
class Random_MapsAxisFeatureGenerator(CvMapGeneratorUtil.FeatureGenerator):
	def getLatitudeAtPlot(self, iX, iY):
		"returns a value in the range of 0.0 (tropical) to 1.0 (polar)"
		# SC 03-NOV-2005
		# Changed this to match the one in CvMapGeneratorUtil
		return abs((self.iGridH/2) - iY)/float(self.iGridH/2) # 0.0 = equator, 1.0 = pole

def addFeatures():
	NiTextOut("Adding Features (Python Random_Maps Axis) ...")
	featuregen = Random_MapsAxisFeatureGenerator()
	featuregen.addFeatures()
	return 0
